//////////////////////////////////////////////////////////////////////////////
//
//    BORDERIMAGESDIALOG.H
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////


#ifndef BORDERIMAGESDIALOG_H
#define BORDERIMAGESDIALOG_H

// Include files for Qt

#include <qcolor.h>
#include <qstring.h>

// Include files for KDE

#include <kdialogbase.h>

// Digikam includes

#include <digikam/albuminfo.h>

#define NO_PROCESS       0
#define UNDER_PROCESS    1
#define PROCESS_DONE     2
#define STOP_PROCESS     3

class QLabel;
class QComboBox;
class QPushButton;
class QListViewItemIterator;

class KProcess;
class KConfig;
class KProgress;
class KListView;

class ImageFileBorderItem;

class BorderImagesDialog : public KDialogBase
{
Q_OBJECT

 public:
   BorderImagesDialog(QWidget *parent=0, QStringList filesList=0);
   ~BorderImagesDialog();

 private slots:
   void slotAbout(void);
   void slotProcessStart(void);
   void slotOk();
   void slotPreview(void);
   void PreviewProcessDone(KProcess* proc);
   void slotProcessStop();
   void ProcessDone(KProcess* proc);
   void slotListDoubleClicked(QListViewItem *itemClicked);
   void slotOptionsClicked(void);
   void slotReadStd(KProcess* proc, char *buffer, int buflen);
   void slotPreviewReadStd(KProcess* proc, char *buffer, int buflen);
   void closeEvent ( QCloseEvent *e );
   void slotPreviewStop( void );

 protected:
   QLabel                *m_labelBorderType;
   QComboBox             *m_borderType;
   KListView             *m_listFiles;
   KProgress             *m_progress;
   QPushButton           *m_optionsButton;
   QPushButton           *m_previewButton;
   QLabel                *m_statusbar;

   int                    m_solidWidth;
   QColor                 m_solidColor;

   int                    m_lineNiepceWidth;
   QColor                 m_lineNiepceColor;
   int                    m_NiepceWidth;
   QColor                 m_NiepceColor;

   int                    m_raiseWidth;

   int                    m_frameWidth;
   int                    m_bevelWidth;
   QColor                 m_frameColor;

   int                    m_processStatus;
   int                    m_progressStatus;
   KConfig               *m_config;
   QStringList            m_selectedImageFiles;
   int                    m_nbItem;
   KProcess              *m_ProcessusProc;
   KProcess              *m_PreviewProc;
   QListViewItemIterator *m_listFile2Process_iterator;
   QString                m_commandLine;
   QString                m_tmpFolder;
   QString                m_previewOutput;

   void listImageFiles(void);
   bool startProcess(void);
   void endProcess(void);
   QString makeProcess(KProcess* proc, Digikam::AlbumInfo *album, ImageFileBorderItem *item);
   void processAborted(void);
   void endPreview(void);
};

#endif  // BORDERIMAGESDIALOG_H
