//////////////////////////////////////////////////////////////////////////////
//
//    EFFECTIMAGESDIALOG.H
//
//    Copyright (C) 2004 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////


#ifndef EFFECTIMAGESDIALOG_H
#define EFFECTIMAGESDIALOG_H

// Include files for Qt

#include <qstring.h>

// Include files for KDE

#include <kdialogbase.h>

// Digikam includes

#include <digikam/albuminfo.h>

#define NO_PROCESS       0
#define UNDER_PROCESS    1
#define PROCESS_DONE     2
#define STOP_PROCESS     3

class QLabel;
class QComboBox;
class QCheckBox;
class QPushButton;
class QListViewItemIterator;

class KProcess;
class KConfig;
class KProgress;
class KListView;

class ImageFileEffectItem;

class EffectImagesDialog : public KDialogBase
{
Q_OBJECT

 public:
   EffectImagesDialog(QWidget *parent=0, QStringList filesList=0);
   ~EffectImagesDialog();

 private slots:
   void slotAbout(void);
   void slotProcessStart(void);
   void slotOk();
   void slotPreview(void);
   void PreviewProcessDone(KProcess* proc);
   void slotProcessStop();
   void ProcessDone(KProcess* proc);
   void slotListDoubleClicked(QListViewItem *itemClicked);
   void slotOptionsClicked(void);
   void slotReadStd(KProcess* proc, char *buffer, int buflen);
   void slotPreviewReadStd(KProcess* proc, char *buffer, int buflen);
   void slotEffectTypeChanged(const QString &string);
   void closeEvent ( QCloseEvent *e );
   void slotPreviewStop( void );

 protected:
   QLabel                *m_labelEffectType;
   QComboBox             *m_EffectType;
   QCheckBox             *m_smallPreview;
   KListView             *m_listFiles;
   KProgress             *m_progress;
   QPushButton           *m_optionsButton;
   QPushButton           *m_previewButton;
   QLabel                *m_statusbar;

   int                    m_latWidth;
   int                    m_latHeight;
   int                    m_latOffset;
   int                    m_charcoalRadius;
   int                    m_charcoalDeviation;
   int                    m_edgeRadius;
   int                    m_embossRadius;
   int                    m_embossDeviation;
   int                    m_implodeFactor;
   int                    m_paintRadius;
   int                    m_shadeAzimuth;
   int                    m_shadeElevation;
   int                    m_solarizeFactor;
   int                    m_spreadRadius;
   int                    m_swirlDegrees;
   int                    m_waveAmplitude;
   int                    m_waveLenght;

   int                    m_processStatus;
   int                    m_progressStatus;
   KConfig               *m_config;
   QStringList            m_selectedImageFiles;
   int                    m_nbItem;
   KProcess              *m_ProcessusProc;
   KProcess              *m_PreviewProc;
   QListViewItemIterator *m_listFile2Process_iterator;
   QString                m_commandLine;
   QString                m_tmpFolder;
   QString                m_previewOutput;

   void listImageFiles(void);
   bool startProcess(void);
   void endProcess(void);
   QString makeProcess(KProcess* proc, Digikam::AlbumInfo *album, ImageFileEffectItem *item);
   void processAborted(void);
   void endPreview(void);
};

#endif  // EFFECTIMAGESDIALOG_H
