//////////////////////////////////////////////////////////////////////////////
//
//    RECOMPRESSIMAGESDIALOG.CPP
//
//    Copyright (C) 2004 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qprocess.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpalette.h>

// Include files for KDE

#include <klocale.h>
#include <kprogress.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kimageio.h>
#include <kprocess.h>
#include <klineeditdlg.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/albuminfo.h>
#include <digikam/digikamio.h>

// Local includes

#include "recompressimagesdialog.h"
#include "recompressoptionsdialog.h"
#include "outputdialog.h"


class ImageFileRecompressItem : public KListViewItem
{
public:
    ImageFileRecompressItem(QListView * parent, QString const & nameSrc,
                         QString const & nameDest, QString const & result)
                       : KListViewItem( parent, nameSrc, nameDest, result),
                         _nameSrc(nameSrc), _nameDest(nameDest), _result(result)
    {}

    QString nameSrc()                   { return _nameSrc; }
    QString nameDest()                  { return _nameDest; }
    QString result()                    { return _result; }
    QString error()                     { return _error; }
    QString outputMess()                { return _outputMess; }

    void changeResult(QString text)     { setText(2, text); }
    void changeError(QString text)      { _error = text; }
    void changeNameDest(QString text)   { _nameDest = text; setText(1, _nameDest); }
    void changeOutputMess(QString text) { _outputMess.append(text); }

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(2) != i18n("OK") && text(2) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(2) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString _nameSrc;
    QString _nameDest;
    QString _result;
    QString _error;
    QString _outputMess;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

RecompressImagesDialog::RecompressImagesDialog(QWidget *parent, QStringList filesList)
                 : KDialogBase( parent, "RecompressImagesDialog", true,
                   i18n("Batch Recompress Images options"), Help|User1|User2|Cancel,
                   Cancel, true, i18n("&About"), i18n("&Start"))
{
    m_convertStatus = NO_PROCESS;
    m_progressStatus = 0;
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_convertProc = 0L;

    setHelp("plugin-recompressimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Recompress images options"), box );

    m_optionsButton = new QPushButton (groupBox1, "OptionButton");
    m_optionsButton->setText(i18n("Options"));
    QWhatsThis::add( m_optionsButton, i18n("<p>You can choose here the image files format "
                                           "options using for the recompress process."));

    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox2 = new QGroupBox( 2, Qt::Horizontal, i18n("Files operations"), box );

    m_labelOverWrite = new QLabel (i18n("Overwrite mode:"), groupBox2);
    m_overWriteMode = new QComboBox( false, groupBox2 );
    m_overWriteMode->insertItem(i18n("Rename"));
    m_overWriteMode->insertItem(i18n("Ask"));
    m_overWriteMode->insertItem(i18n("Skip"));
    m_overWriteMode->insertItem(i18n("Always OverWrite"));
    m_overWriteMode->setCurrentText (i18n("Ask"));
    QWhatsThis::add( m_overWriteMode, i18n("<p>Select here the overwrite mode if your targets image "
                                           "files already exist.") );

    m_removeOriginal = new QCheckBox(i18n("Remove original"), groupBox2);
    QWhatsThis::add( m_removeOriginal, i18n("<p>If you enable this option, "
                                            "all originals images files will be removed.") );
    m_removeOriginal->setChecked( false );

    dvlay->addWidget( groupBox2 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 2, Qt::Horizontal, i18n("Target Album"), box );

    m_albumList = new QComboBox( false, groupBox3 );
    QStringList albumsList;
    for (Digikam::AlbumInfo *album=Digikam::AlbumManager::instance()->firstAlbum() ;
         album ; album = album->nextAlbum())
        {
        album->openDB();
        albumsList.append (album->getTitle());
        album->closeDB();
        }

    albumsList.sort();
    m_albumList->insertStringList(albumsList);
    m_albumList->setCurrentText (Digikam::AlbumManager::instance()->currentAlbum()->getTitle());
    QWhatsThis::add( m_albumList, i18n("<p>Select here the target Album for the recompressing process.") );

    m_addNewAlbumButton = new QPushButton (groupBox3, "PushButton_AddNewAlbum");
    m_addNewAlbumButton->setText(i18n( "&Add new album") );
    m_addNewAlbumButton->setAutoRepeat( false );
    QWhatsThis::add( m_addNewAlbumButton, i18n( "Add a new Album in the Digikam Albums library."));

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    QGroupBox * groupBox4 = new QGroupBox( 1, Qt::Horizontal, i18n("Images to recompress (%1 items)")
                                                              .arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox4 );
    m_listFiles->addColumn(i18n("Source image"));
    m_listFiles->addColumn(i18n("Target image"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(2);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 150 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here the operations results during the recompression. "
                                       "Double click on one item for more informations after the process.") );

    m_statusbar = new QLabel( groupBox4, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>The current actived task..") );

    m_progress = new KProgress( groupBox4, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox4 );

    //---------------------------------------------

    connect(m_addNewAlbumButton, SIGNAL(clicked()),
            this, SLOT(slotAddNewAlbum()));

    connect(m_listFiles, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotListDoubleClicked(QListViewItem *)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));

    connect(m_optionsButton, SIGNAL(clicked()),
            this, SLOT(slotOptionsFileFormatClicked()));

    connect(m_overWriteMode, SIGNAL(activated(const QString &)),
            this, SLOT(slotOverWriteModeActived(const QString &)));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("RecompressImages Settings");

    m_JPEGCompression = m_config->readNumEntry("JPEGCompression", 75);
    m_PNGCompression = m_config->readNumEntry("PNGCompression", 75);
    m_TIFFCompressionAlgo = m_config->readEntry("TIFFCompressionAlgo", i18n("None"));
    m_TGACompressionAlgo = m_config->readEntry("TGACompressionAlgo", i18n("None"));

    m_overWriteMode->setCurrentItem(m_config->readNumEntry("OverWriteMode", 0));  // 'Ask' per default...

    if (m_config->readEntry("RemoveOriginal", "true") == "true")
        m_removeOriginal->setChecked( true );
    else
        m_removeOriginal->setChecked( false );

    delete m_config;

    //---------------------------------------------

    listImageFiles();
    slotOverWriteModeActived(m_overWriteMode->currentText());
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

RecompressImagesDialog::~RecompressImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////


void RecompressImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch recompress images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n\n"
                                  "This plugin use the \"convert\" program from \"ImageMagick\" package.\n"),
                                  i18n("About Digikam batch recompress images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotOverWriteModeActived( const QString &string )
{
    if ( string == i18n("Always OverWrite") )
       {
       m_removeOriginal->setEnabled(false);
       m_removeOriginal->setChecked(false);
       }
    else
       m_removeOriginal->setEnabled(true);
}


/////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotAddNewAlbum( void )
{
    QDir libraryDir( Digikam::AlbumManager::instance()->getLibraryPath());

    if (!libraryDir.exists())
        {
        KMessageBox::error(0, i18n("Album Library has not been set correctly\n"
                                   "Please run Setup"));
        return;
        }

    bool ok;
    m_newDir = KLineEditDlg::getText(i18n("Enter New Album Name: "), "", &ok, this);
    if (!ok) return;

    KURL newAlbumURL(Digikam::AlbumManager::instance()->getLibraryPath());
    newAlbumURL.addPath(m_newDir);

    KIO::SimpleJob* job = KIO::mkdir(newAlbumURL);
    connect(job, SIGNAL(result(KIO::Job*)),
            this, SLOT(slot_onAlbumCreate(KIO::Job*)));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slot_onAlbumCreate(KIO::Job* job)
{
    if (job->error())
        {
        job->showErrorDialog(this);
        }
    else
        {
        m_albumList->insertStringList(m_newDir, -1);
        m_albumList->setCurrentText (m_newDir);
        }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotOptionsFileFormatClicked(void)
{
    RecompressOptionsDialog *optionsDialog = new RecompressOptionsDialog(0);

    optionsDialog->m_JPEGCompression->setValue(m_JPEGCompression);
    optionsDialog->m_compressLossLess->setChecked(m_compressLossLess);
    optionsDialog->m_PNGCompression->setValue(m_PNGCompression);
    optionsDialog->m_TIFFCompressionAlgo->setCurrentText(m_TIFFCompressionAlgo);
    optionsDialog->m_TGACompressionAlgo->setCurrentText(m_TGACompressionAlgo);

    if ( optionsDialog->exec() == KMessageBox::Ok )
       {
       m_JPEGCompression = optionsDialog->m_JPEGCompression->value();
       m_compressLossLess = optionsDialog->m_compressLossLess->isChecked();
       m_PNGCompression = optionsDialog->m_PNGCompression->value();
       m_TIFFCompressionAlgo = optionsDialog->m_TIFFCompressionAlgo->currentText();
       m_TGACompressionAlgo = optionsDialog->m_TGACompressionAlgo->currentText();
       }

    delete optionsDialog;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotProcessStart( void )
{
    if ( m_removeOriginal->isChecked() == true )
        {
        if ( KMessageBox::warningContinueCancel(0,
             i18n("All original images files will be removed from the Album!\nDo you want to continue?"),
             i18n("Delete original images files"), KStdGuiItem::cont(),
             "Digikamplugin-RecompressImages-AlwaysRemomveOriginalFiles") != KMessageBox::Continue )
           return;
        }

    m_statusbar->setText(i18n("Process under progress. Please wait..."));
    m_convertStatus = UNDER_PROCESS;
    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotStop()));

    m_optionsButton->setEnabled(false);
    m_labelOverWrite->setEnabled(false);
    m_overWriteMode->setEnabled(false);
    m_removeOriginal->setEnabled(false);
    m_addNewAlbumButton->setEnabled(false);
    m_albumList->setEnabled(false);

    m_listFile2Process_iterator = new QListViewItemIterator( m_listFiles );
    startProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool RecompressImagesDialog::startProcess(void)
{
    if ( m_convertStatus == STOP_PROCESS )
       {
       endProcess();
       return true;
       }

    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileRecompressItem *item = static_cast<ImageFileRecompressItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->setCurrentItem(item);

    QString imageExt = item->nameSrc().section('.', -1 );

    if (imageExt != "JPEG" && imageExt != "jpeg" && imageExt != "JPG" && imageExt != "jpg" &&
        imageExt != "PNG" && imageExt != "png" &&
        imageExt != "TIFF" && imageExt != "tiff" && imageExt != "TIF" && imageExt != "tif" &&
        imageExt != "TGA" && imageExt != "tga")
        {
        item->changeResult(i18n("Skipped !"));
        item->changeError(i18n("image file format unsupported!"));
        ++*m_listFile2Process_iterator;
        ++m_progressStatus;
        m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

        if ( m_listFile2Process_iterator->current() )
           {
           startProcess();
           return true;
           }
        else
           {
           endProcess();
           return true;
           }
        }

    KURL desturl(targetAlbum->getPath() + "/" + item->nameDest());

    if ( KIO::NetAccess::exists(desturl) == true )
       {
       switch (overwriteMode())
          {
          case OVERWRITE_ASK:
             {
             int ValRet = KMessageBox::warningYesNoCancel(0,
                          i18n("The destination file \"%1\" already exists!\n"
                          "Do you want overwrite it?").arg(item->nameDest()),
                          i18n("Overwrite destination image file"), KStdGuiItem::cont());

             if ( ValRet == KMessageBox::No )
                {
                item->changeResult(i18n("Skipped !"));
                item->changeError(i18n("destination image file already exists (skipped by user)!"));
                ++*m_listFile2Process_iterator;
                ++m_progressStatus;
                m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

                if ( m_listFile2Process_iterator->current() )
                   {
                   startProcess();
                   return true;
                   }
                else
                   {
                   endProcess();
                   return true;
                   }
                }

             if ( ValRet == KMessageBox::Cancel )
                {
                endProcess();
                return false;
                }

             break;
             }

          case OVERWRITE_RENAME:
             {
             QFileInfo *Target = new QFileInfo(targetAlbum->getPath() + "/" + item->nameDest());
             QString newFileName = RenameTargetImageFile(Target);

             if ( newFileName == QString::null )
                {
                item->changeResult(i18n("Failed !"));
                item->changeError(i18n("destination image file already exists and cannot be renamed!"));
                ++*m_listFile2Process_iterator;
                ++m_progressStatus;
                m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

                if ( m_listFile2Process_iterator->current() )
                   {
                   startProcess();
                   return true;
                   }
                else
                   {
                   endProcess();
                   return true;
                   }
                }
             else
                {
                QFileInfo *newTarget = new QFileInfo(newFileName);
                item->changeNameDest(newTarget->fileName());
                }

             break;
             }

          case OVERWRITE_SKIP:
             {
             item->changeResult(i18n("Skipped !"));
             item->changeError(i18n("destination image file already exists (skipped automaticly)!"));
             ++*m_listFile2Process_iterator;
             ++m_progressStatus;
             m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

             if ( m_listFile2Process_iterator->current() )
                {
                startProcess();
                return true;
                }
             else
                {
                endProcess();
                return true;
                }
             break;
             }

          case OVERWRITE_OVER:   // In this case do nothing : 'convert' default mode...
             break;

          default:
             {
             endProcess();
             return true;
             break;
             }
          }
       }

    QString CommandLine ="convert";
    m_convertProc = new KProcess;
    *m_convertProc << "convert";

    if (imageExt == "JPEG" || imageExt == "jpeg" || imageExt == "JPG" || imageExt == "jpg")
       {
       if (m_compressLossLess == true)
          {
          *m_convertProc << "-compress" << "Lossless";
          CommandLine.append( " -compress Lossless ");
          }
       else
          {
          *m_convertProc << "-quality";
          QString Temp;
          *m_convertProc << Temp.setNum( m_JPEGCompression );
          CommandLine.append( " -quality " + Temp.setNum( m_JPEGCompression ) );
          }
       }

    else if (imageExt == "PNG" || imageExt == "png")
       {
       *m_convertProc << "-quality";
       QString Temp;
       *m_convertProc << Temp.setNum( m_PNGCompression );
       CommandLine.append( " -quality " + Temp.setNum( m_PNGCompression ) );
       }

    else if (imageExt == "TIFF" || imageExt == "tiff" || imageExt == "TIF" || imageExt == "tif")
       {
       *m_convertProc << "-compress";
       if (m_TIFFCompressionAlgo == i18n("None"))
          {
          *m_convertProc << "None";
          CommandLine.append( " -compress None" );
          }
       else
          {
          *m_convertProc << m_TIFFCompressionAlgo;
          CommandLine.append( " -compress " + m_TIFFCompressionAlgo );
          }
       }

    else if (imageExt == "TGA" || imageExt == "tga")
       {
       *m_convertProc << "-compress";
       if (m_TGACompressionAlgo == i18n("None"))
          {
          *m_convertProc << "None";
          CommandLine.append( " -compress None" );
          }
       else
          {
          *m_convertProc << m_TGACompressionAlgo;
          CommandLine.append( " -compress " + m_TGACompressionAlgo );
          }
       }

    else
       return false;                // The image file format isn't supported.

    *m_convertProc << "-verbose";
    *m_convertProc << currentAlbum->getPath() + "/" + item->nameSrc();
    *m_convertProc << targetAlbum->getPath() + "/" + item->nameDest();
    CommandLine.append( " -verbose " + currentAlbum->getPath() + "/" + item->nameSrc() + " "
                        + targetAlbum->getPath() + "/" + item->nameDest() + "\n\n");

    item->changeOutputMess(CommandLine);

    connect(m_convertProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(ProcessDone(KProcess*)));

    connect(m_convertProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    connect(m_convertProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    bool result = m_convertProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'convert' program from 'ImageMagick' package.\n"
                          "Please, check your installation!"));
       return false;
       }

    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotReadStd(KProcess* proc, char *buffer, int buflen)
{
    ImageFileRecompressItem *item = static_cast<ImageFileRecompressItem*>( m_listFile2Process_iterator->current() );
    item->changeOutputMess( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::ProcessDone(KProcess* proc)
{
    int ValRet = proc->exitStatus();
    qDebug ("Convert exit (%i)", ValRet);

    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileRecompressItem *item = static_cast<ImageFileRecompressItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());

    switch (ValRet)
       {
       case 0:  // Process finished successfully !
          {
          item->changeResult(i18n("OK"));
          item->changeError(i18n("no recompression error"));

          // Save the comments for the converted image.

          currentAlbum->openDB();
          QString comments = currentAlbum->getItemComments(item->nameSrc());
          currentAlbum->closeDB();

          targetAlbum->openDB();
          targetAlbum->setItemComments(item->nameDest(), comments);
          targetAlbum->closeDB();

          if ( m_removeOriginal->isChecked() == true )
             {
             KURL deleteImage(currentAlbum->getPath() + "/" + item->nameSrc());
             if ( KIO::NetAccess::del(deleteImage) == false )
                {
                item->changeResult(i18n("Warning !"));
                item->changeError(i18n("cannot remove original image file!"));
                }
             }
          break;
          }
       case 15: //  process aborted !
          {
          processAborted();
          break;
          }
       default : // Processing error !
          {
          item->changeResult(i18n("Failed !!!"));
          item->changeError(i18n("cannot process original image file!"));
          break;
          }
       }

    ++*m_listFile2Process_iterator;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if ( m_listFile2Process_iterator->current() )
       startProcess();
    else
       {
       m_statusbar->setText(i18n("Process finished!"));
       endProcess();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("RecompressImages Settings");
    m_config->writeEntry("JPEGCompression", m_JPEGCompression);
    m_config->writeEntry("PNGCompression", m_PNGCompression);
    m_config->writeEntry("CompressLossLess", m_compressLossLess);
    m_config->writeEntry("TIFFCompressionAlgo", m_TIFFCompressionAlgo);
    m_config->writeEntry("TGACompressionAlgo", m_TGACompressionAlgo);
    m_config->writeEntry("OverWriteMode", m_overWriteMode->currentItem());
    m_config->writeEntry("RemoveOriginal", m_removeOriginal->isChecked());
    m_config->sync();

    delete m_config;
    delete this;
}


////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void RecompressImagesDialog::listImageFiles(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, QDir::Name);
    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileRecompressItem *item = new ImageFileRecompressItem(m_listFiles,
                                                                fi->fileName(),
                                                                fi->fileName(),
                                                                ""
                                                                );
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotListDoubleClicked(QListViewItem *itemClicked)
{
    ImageFileRecompressItem *item = static_cast<ImageFileRecompressItem*>( itemClicked );

    if (m_convertStatus == PROCESS_DONE)
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Recompress image error"),
                                                   item->outputMess(),
                                                   i18n("Image \"%1\": %2\n\nThe ouput messages are:\n")
                                                        .arg(item->nameSrc()).arg(item->error())
                                                   );
       infoDialog->exec();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::slotStop( void )
{
    // Try to kill the current process !
    if ( m_convertProc->isRunning() == true ) m_convertProc->kill(SIGTERM);

    // If kill operation failed, Stop the process at the next image !
    if ( m_convertStatus == UNDER_PROCESS ) m_convertStatus = STOP_PROCESS;

    processAborted();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

int RecompressImagesDialog::overwriteMode(void)
{
    QString OverWrite = m_overWriteMode->currentText();

    if (OverWrite == i18n("Ask"))
        return OVERWRITE_ASK;

    if (OverWrite == i18n("Rename"))
        return OVERWRITE_RENAME;

    if (OverWrite == i18n("Skip"))
        return OVERWRITE_SKIP;

    if (OverWrite == i18n("Always OverWrite"))
        return OVERWRITE_OVER;

    return OVERWRITE_ASK;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::processAborted(void)
{
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileRecompressItem *item = static_cast<ImageFileRecompressItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());
    m_statusbar->setText(i18n("Process aborted by user!"));
    
    item->changeResult(i18n("Aborted !"));
    item->changeError(i18n("process aborted by user"));
    KURL deleteImage(targetAlbum->getPath() + "/" + item->nameDest());
    KIO::NetAccess::del(deleteImage);                              // Try to delete de destination image !
    endProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RecompressImagesDialog::endProcess(void)
{
    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumManager::instance()->refreshItemHandler(currentAlbum->getTitle());
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    Digikam::AlbumManager::instance()->refreshItemHandler(targetAlbum->getTitle());
    m_convertStatus = PROCESS_DONE;
    setButtonText( User2, i18n("&Close") );

    disconnect(this, SIGNAL(user2Clicked()), this, SLOT(slotStop()));

    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotOk()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString RecompressImagesDialog::RenameTargetImageFile(QFileInfo *fi)
{
    QString Temp;
    int Enumerator = 0;
    KURL NewDestUrl;

    do
       {
       ++Enumerator;
       Temp = Temp.setNum( Enumerator );
       NewDestUrl = fi->filePath().left( fi->filePath().findRev('.', -1)) + "_" + Temp
                    + "." + fi->filePath().section('.', -1 );
       }
    while ( Enumerator < 100 && KIO::NetAccess::exists(NewDestUrl) == true );

    if (Enumerator == 100) return QString::null;

    return (NewDestUrl.path());
}

