//////////////////////////////////////////////////////////////////////////////
//
//    RENAMEIMAGESDIALOG.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qcolor.h>

// Include files for KDE

#include <klocale.h>
#include <klineedit.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <knuminput.h>
#include <kdebug.h>
#include <kprogress.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <klineeditdlg.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/albuminfo.h>
#include <digikam/digikamio.h>

// Locale includes

#include "renameimagesdialog.h"


class ImageFileRenameItem : public KListViewItem
{
public:
    ImageFileRenameItem(QListView * parent, QString const & nameSrc, QString const & nameDest,
                        QString const & result)
        : KListViewItem( parent, nameSrc, nameDest, result), _nameSrc(nameSrc),
                        _nameDest(nameDest), _result(result)
    {}

    QString nameSrc()               { return _nameSrc; }
    QString nameDest()              { return _nameDest; }
    QString result()                { return _result; }
    void changeResult(QString text) {setText(2, text);}

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(2) != i18n("OK") && text(2) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(2) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString         _nameSrc;
    QString         _nameDest;
    QString         _result;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

RenameImagesDialog::RenameImagesDialog(QWidget *parent, QStringList filesList)
                  : KDialogBase( parent, "RenameImagesDialog", true,
                                 i18n("Batch Rename Images options"), Help|User1|User2|Cancel, Cancel,
                                 true, i18n("&About"), i18n("&Start"))
{
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_progressStatus = 0;

    setHelp("plugin-renameimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Template to use for renaming images"), box );

    m_labelPrefix = new QLabel( i18n("Prefix string:"), groupBox1);
    m_prefix = new KLineEdit(groupBox1);
    QWhatsThis::add( m_prefix, i18n("<p>Enter here the prefix numeric enumerator string.") );

    m_labelStart = new QLabel( i18n("Numeric enumerator start value:"), groupBox1);
    m_enumeratorStart = new KIntSpinBox( 1, 100000, 1, 1, 10, groupBox1);
    QWhatsThis::add( m_enumeratorStart, i18n("<p>Enter here the first numeric enumerator value (1 to 100000).") );

    m_addOriginalFileName = new QCheckBox(i18n("Add original file name"), groupBox1);
    QWhatsThis::add( m_addOriginalFileName, i18n("<p>If you enable this option, "
                     "the original file name will added in the new file name.") );
    m_addOriginalFileName->setChecked( false );

    m_addImageFileDate = new QCheckBox(i18n("Add file date"), groupBox1);
    QWhatsThis::add( m_addImageFileDate, i18n("<p>If you enable this option, "
                     "the last modified image file date will added in the new file name (format: DDMMYYYY).") );
    m_addImageFileDate->setChecked( false );

    m_addImageFileSize = new QCheckBox(i18n("Add file size"), groupBox1);
    QWhatsThis::add( m_addImageFileSize, i18n("<p>If you enable this option, "
                     "the image file size will added in the new file name.") );
    m_addImageFileSize->setChecked( false );

    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox2 = new QGroupBox( 2, Qt::Horizontal, i18n("File list order"), box );

    m_labelSort = new QLabel (i18n("Sort original list:"), groupBox2);
    m_sortList = new QComboBox( false, groupBox2 );
    m_sortList->insertItem(i18n("by file Name"));
    m_sortList->insertItem(i18n("by file Size"));
    m_sortList->insertItem(i18n("by modification Date"));
    m_sortList->setCurrentText (i18n("by file Name"));
    QWhatsThis::add( m_sortList, i18n("<p>Select here the original files list sorting method.") );

    m_reverseOrder = new QCheckBox(i18n("Reverse order"), groupBox2);
    QWhatsThis::add( m_reverseOrder, i18n("<p>If you enable this option, "
                     "all images files list will used in reverse order.") );
    m_reverseOrder->setChecked( false );

    dvlay->addWidget( groupBox2 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 2, Qt::Horizontal, i18n("Target Album"), box );

    m_albumList = new QComboBox( false, groupBox3 );
    QStringList albumsList;
    for (Digikam::AlbumInfo *album=Digikam::AlbumManager::instance()->firstAlbum() ; album ; album = album->nextAlbum())
        {
        album->openDB();
        albumsList.append (album->getTitle());
        album->closeDB();
        }

    albumsList.sort();
    m_albumList->insertStringList(albumsList);
    m_albumList->setCurrentText (Digikam::AlbumManager::instance()->currentAlbum()->getTitle());
    QWhatsThis::add( m_albumList, i18n("<p>Select here the target Album for the renaming process.") );

    m_addNewAlbumButton = new QPushButton (groupBox3, "PushButton_AddNewAlbum");
    m_addNewAlbumButton->setText(i18n( "&Add new album") );
    m_addNewAlbumButton->setAutoRepeat( false );
    QWhatsThis::add( m_addNewAlbumButton, i18n( "Add a new Album in the Digikam Albums library."));

    m_deleteOrignalFiles = new QCheckBox(i18n("Delete original files"), groupBox3);
    QWhatsThis::add( m_deleteOrignalFiles, i18n("<p>If the target Album is different of the current Album, "
                                                "you can delete the original files with this option.") );
    m_deleteOrignalFiles->setChecked( false );

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    QGroupBox * groupBox4 = new QGroupBox( 1, Qt::Horizontal, i18n("New files names (%1 items)").arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox4 );
    m_listFiles->addColumn(i18n("Old name"));
    m_listFiles->addColumn(i18n("New name"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(2);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 150 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here a preview of renaming process and "
                                       "the operation result during this processus.") );

    m_statusbar = new QLabel( groupBox4, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>This is the current active task.") );

    m_progress = new KProgress( groupBox4, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox4 );

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("RenameImages Settings");

    m_prefix->setText(m_config->readEntry("PrefixString", ""));

    if (m_config->readEntry("AddOriginalFileName", "true") == "true")
        m_addOriginalFileName->setChecked( true );
    else
        m_addOriginalFileName->setChecked( false );

    if (m_config->readEntry("AddImageFileDate", "true") == "true")
        m_addImageFileDate->setChecked( true );
    else
        m_addImageFileDate->setChecked( false );

    if (m_config->readEntry("AddImageFileSize", "true") == "true")
        m_addImageFileSize->setChecked( true );
    else
        m_addImageFileSize->setChecked( false );

    if (m_config->readEntry("RemoveoriginalFiles", "true") == "false")
        m_deleteOrignalFiles->setChecked( true );
    else
        m_deleteOrignalFiles->setChecked( false );

    m_enumeratorStart->setValue(m_config->readNumEntry("FirstRenameValue", 1));

    m_sortList->setCurrentItem(m_config->readNumEntry("SortMethod", 0));    // Sort by Name method by default...

    if (m_config->readEntry("ReverseOrder", "true") == "true")
        m_reverseOrder->setChecked( true );
    else
        m_reverseOrder->setChecked( false );

    //---------------------------------------------

    connect(m_addNewAlbumButton, SIGNAL(clicked()),
            this, SLOT(slotAddNewAlbum()));

    connect(m_albumList, SIGNAL(activated(const QString &)),
            this, SLOT(slotAlbumChanged(const QString &)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotRename()));

    connect(m_prefix, SIGNAL(textChanged(const QString &)),
            this, SLOT(slotTemplateSortListChanged(const QString &)));

    connect(m_enumeratorStart, SIGNAL(valueChanged(int)),
            this, SLOT(slotTemplateSortListChanged(int)));

    connect(m_sortList, SIGNAL(activated(int)),
            this, SLOT(slotTemplateSortListChanged(int)));

    connect(m_reverseOrder, SIGNAL(toggled (bool)),
            this, SLOT(slotTemplateSortListChanged(bool)));

    connect(m_addOriginalFileName, SIGNAL(toggled (bool)),
            this, SLOT(slotTemplateSortListChanged(bool)));

    connect(m_addImageFileDate, SIGNAL(toggled (bool)),
            this, SLOT(slotTemplateSortListChanged(bool)));

    connect(m_addImageFileSize, SIGNAL(toggled (bool)),
            this, SLOT(slotTemplateSortListChanged(bool)));

    sortingImageFiles();
    slotAlbumChanged(m_albumList->currentText());

    delete m_config;
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

RenameImagesDialog::~RenameImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////


void RenameImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch rename images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n"),
                                  i18n("About Digikam batch rename images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotAlbumChanged(const QString &albumName)
{
    if ( Digikam::AlbumManager::instance()->currentAlbum()->getTitle() != albumName )
       m_deleteOrignalFiles->setEnabled(true);
    else
       m_deleteOrignalFiles->setEnabled(false);
}


/////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotAddNewAlbum( void )
{
    QDir libraryDir( Digikam::AlbumManager::instance()->getLibraryPath());
    if (!libraryDir.exists())
        {
        KMessageBox::error(0, i18n("Album Library has not been set correctly\n"
                                   "Please run Setup"));
        return;
        }

    bool ok;
    m_newDir = KLineEditDlg::getText(i18n("Enter New Album Name: "), "", &ok, this);
    if (!ok) return;

    KURL newAlbumURL(Digikam::AlbumManager::instance()->getLibraryPath());
    newAlbumURL.addPath(m_newDir);

    KIO::SimpleJob* job = KIO::mkdir(newAlbumURL);

    connect(job, SIGNAL(result(KIO::Job*)),
            this, SLOT(slot_onAlbumCreate(KIO::Job*)));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slot_onAlbumCreate(KIO::Job* job)
{
    if (job->error())
        {
        job->showErrorDialog(this);
        }
    else
        {
        m_albumList->insertStringList(m_newDir, -1);
        m_albumList->setCurrentText (m_newDir);
        }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotRename( void )
{
    m_statusbar->setText(i18n("Process under progress. Please wait..."));

    disconnect( this, SIGNAL(user2Clicked()),
                this, SLOT(slotRename()));

    showButtonCancel( false );
    enableButton(User2, false);

    m_labelPrefix->setEnabled(false);
    m_prefix->setEnabled(false);
    m_labelStart->setEnabled(false);
    m_enumeratorStart->setEnabled(false);
    m_addOriginalFileName->setEnabled(false);
    m_addImageFileDate->setEnabled(false);
    m_addImageFileSize->setEnabled(false);
    m_labelSort->setEnabled(false);
    m_sortList->setEnabled(false);
    m_reverseOrder->setEnabled(false);
    m_deleteOrignalFiles->setEnabled(false);
    m_addNewAlbumButton->setEnabled(false);
    m_albumList->setEnabled(false);

    m_filelist_it = new QListViewItemIterator( m_listFiles );

    if ( m_albumList->currentText() != Digikam::AlbumManager::instance()->currentAlbum()->getTitle() )
        copyItemOperations();  // Copy and rename in another Album.
    else
        {                      // Rename in the current Album.
        while ( m_filelist_it->current() )
            {
            ImageFileRenameItem *item = static_cast<ImageFileRenameItem*>( m_filelist_it->current() );
            m_listFiles->setCurrentItem(item);

            bool ValRet = DigikamIO::rename( Digikam::AlbumManager::instance()->currentAlbum(),
                                             item->nameSrc(),
                                             item->nameDest() );

            if (ValRet == true )
               item->changeResult(i18n("OK"));
            else
               item->changeResult(i18n("Failed !!!"));

            ++*m_filelist_it;
            ++m_progressStatus;
            m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));
            }

        m_statusbar->setText(i18n("Process finished!"));
        enableButton(User2, true);
        setButtonText( User2, i18n("&Close") );

        connect(this, SIGNAL(user2Clicked()),
                this, SLOT(slotOk()));
        }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::copyItemOperations(void)
{
    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileRenameItem *item = static_cast<ImageFileRenameItem*>( m_filelist_it->current() );
    m_listFiles->setCurrentItem(item);

    KIO::CopyJob* job = KIO::copy(currentAlbum->getPath() + "/" + item->nameSrc(),
                                  targetAlbum->getPath() + "/" +  item->nameDest(),
                                  true);

    connect(job, SIGNAL(result(KIO::Job*)),
            this, SLOT(slotResult(KIO::Job*)));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotResult( KIO::Job *job )
{
    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileRenameItem *item = static_cast<ImageFileRenameItem*>( m_filelist_it->current() );

    if (job->error())
        item->changeResult(i18n("Failed !!!"));
    else
        {
        // Copy the image comments.

        currentAlbum->openDB();
        QString comments = currentAlbum->getItemComments(item->nameSrc());
        currentAlbum->closeDB();
        targetAlbum->openDB();
        targetAlbum->setItemComments(item->nameDest(), comments);
        targetAlbum->closeDB();

        if ( m_deleteOrignalFiles->isChecked() == true )
           {
           KURL deleteImage(currentAlbum->getPath() + "/" + item->nameSrc());
           if ( KIO::NetAccess::del(deleteImage) == false )
                item->changeResult(i18n("Cannot delete original !"));
           else
                item->changeResult(i18n("OK"));
           }
        else
           item->changeResult(i18n("OK"));
        }

    ++*m_filelist_it;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if (m_filelist_it->current())     // Next image ?
        copyItemOperations();
    else                              // Copy is done...
        {
        m_statusbar->setText(i18n("Process finished!"));
        enableButton(User2, true);
        setButtonText( User2, i18n("&Close") );
        connect(this, SIGNAL(user2Clicked()), this, SLOT(slotOk()));
        }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("RenameImages Settings");
    m_config->writeEntry("PrefixString", m_prefix->text());
    m_config->writeEntry("AddOriginalFileName", m_addOriginalFileName->isChecked());
    m_config->writeEntry("AddImageFileDate", m_addImageFileDate->isChecked());
    m_config->writeEntry("AddImageFileSize", m_addImageFileSize->isChecked());
    m_config->writeEntry("FirstRenameValue", m_enumeratorStart->value());
    m_config->writeEntry("SortMethod", m_sortList->currentItem());
    m_config->writeEntry("ReverseOrder", m_reverseOrder->isChecked());
    m_config->writeEntry("RemoveoriginalFiles", m_deleteOrignalFiles->isChecked());
    m_config->sync();

    delete m_config;
    delete this;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotTemplateSortListChanged(int index)
{
    sortingImageFiles();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotTemplateSortListChanged(bool on)
{
    sortingImageFiles();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void RenameImagesDialog::slotTemplateSortListChanged(const QString &text)
{
    sortingImageFiles();
}


////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void RenameImagesDialog::sortingImageFiles(void)
{
    int sType = sortType();
    int sortMethod;

    switch (sType)
       {
       case SORT_BY_NAME:
          {
          sortMethod = QDir::Name;
          break;
          }

       case SORT_BY_SIZE:
          {
          sortMethod = QDir::Size;
          break;
          }

       case SORT_BY_DATE:
          {
          sortMethod = QDir::Time;
          break;
          }

       default:
          sortMethod = QDir::Name;
          break;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    if (m_reverseOrder->isChecked() == false)
       fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, sortMethod);
    else
       fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, sortMethod | QDir::Reversed );

    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;
    int imageIndex = 0;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileRenameItem *item = new ImageFileRenameItem(m_listFiles,
                                                              fi->fileName(),
                                                              oldFileName2NewFileName(fi, imageIndex),
                                                              ""
                                                              );
          ++imageIndex;
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

int RenameImagesDialog::sortType(void)
{
    QString Sort = m_sortList->currentText();

    if (Sort == i18n("by file Name"))
        return SORT_BY_NAME;

    if (Sort == i18n("by file Size"))
        return SORT_BY_SIZE;

    return SORT_BY_DATE;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString RenameImagesDialog::oldFileName2NewFileName(QFileInfo *fi, int id)
{
    QString Temp, Temp2;
    int nbChar;

    // Fill the new filename index with '0' if necessary.

    Temp2 = Temp2.setNum( m_enumeratorStart->value() + m_nbItem );
    nbChar = Temp2.length();
    Temp2 = Temp2.setNum( id + m_enumeratorStart->value() );
    nbChar = nbChar - Temp2.length();
    Temp = m_prefix->text() + Temp.fill('0', nbChar);
    Temp = Temp + Temp2;

    if (m_addOriginalFileName->isChecked())                                      // Add old filename.
       Temp = Temp + "-" + fi->fileName().left( fi->fileName().findRev('.',-1) );

    if (m_addImageFileDate->isChecked())
       Temp = Temp + "-" + fi->lastModified().toString("ddMMyyyy");              // Add last mofified date of file.

    if (m_addImageFileSize->isChecked())
       Temp = Temp + i18n("-(%1)").arg( KIO::convertSize(fi->size()));           // Add file size.

    Temp = Temp + "." + fi->extension();                                         // Add file extension.

    return Temp;
}
