//////////////////////////////////////////////////////////////////////////////
//
//    RESIZEOPTIONSDIALOG.CPP
//
//    Copyright (C) 2004 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qwhatsthis.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>

// Include files for KDE

#include <klocale.h>
#include <knuminput.h>

// Local includes

#include "resizeoptionsdialog.h"


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

ResizeOptionsDialog::ResizeOptionsDialog(QWidget *parent)
                   : KDialogBase( parent, "ResizeOptionsDialog", true,
                     i18n("Resize Images options"), Ok|Cancel, Ok, false)
{
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );
    QString whatsThis;

    QGroupBox * groupBox1 = new QGroupBox( 1, Qt::Horizontal, i18n("Resize options"), box );

    m_size = new KIntNumInput(640, groupBox1);
    m_size->setRange(10, 5000, 1, true );
    m_size->setLabel( i18n("New size:") );
    QWhatsThis::add( m_size, i18n("<p>The new images size in pixels.") );

    m_label_resizeFilter = new QLabel (i18n("Filter name:"), groupBox1);

    m_resizeFilter = new QComboBox( false, groupBox1 );
    m_resizeFilter->insertItem("Bessel");
    m_resizeFilter->insertItem("Blackman");
    m_resizeFilter->insertItem("Box");
    m_resizeFilter->insertItem("Catrom");
    m_resizeFilter->insertItem("Cubic");
    m_resizeFilter->insertItem("Gaussian");
    m_resizeFilter->insertItem("Hermite");
    m_resizeFilter->insertItem("Hanning");
    m_resizeFilter->insertItem("Hamming");
    m_resizeFilter->insertItem("Lanczos");
    m_resizeFilter->insertItem("Mitchell");
    m_resizeFilter->insertItem("Point");
    m_resizeFilter->insertItem("Quadratic");
    m_resizeFilter->insertItem("Sinc");
    m_resizeFilter->insertItem("Triangle");
    QWhatsThis::add( m_resizeFilter, i18n("<p>Select here the filter name for the resize image process. "
                                          "This filter will be used like a kernel convolution process "
                                          "during the increase image size rendering. The default filter "
                                          "is 'Lanczos'.") );
    m_label_resizeFilter->setBuddy( m_resizeFilter );

    dvlay->addWidget( groupBox1 );

    m_label_size = new QLabel ( i18n("Note: the images will be resized to\n"
                                     "this size. The width or the height of the\n"
                                     "images will be automatically\n"
                                     "selected in depending of the images orientation.\n"
                                     "The images aspect ratio are conserved."), box);
    dvlay->addWidget( m_label_size );

}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

ResizeOptionsDialog::~ResizeOptionsDialog()
{
}


